     //Microsoft Developer Studio generated resource script.
//
// edited
//

#include <windows.h>
#include "resource.h"

/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
FILEVERSION 4,62,0,0
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x21L
#else
 FILEFLAGS 0x20L
#endif
 FILEOS 0x40004L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "CompanyName", "Insecure.Org\0"
            VALUE "FileDescription", "Nmap\0"
            VALUE "FileVersion", "4.62\0"
            VALUE "InternalName", "Nmap\0"
            VALUE "LegalCopyright", "Copyright (c) Insecure.Com LLC (fyodor@insecure.org)\0"
            VALUE "LegalTrademarks", "NMAP\0"
            VALUE "OriginalFilename", "nmap.exe\0"
            VALUE "ProductName", "Nmap\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END

/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
IDI_NMAP                ICON    DISCARDABLE     "icon1.ico"
